from contextlib import contextmanager
from timeit import default_timer
from pathlib import Path

gas_ved_dict = {'Изготовление свай': 'Изготовление металлической сваи Ø219х8 L= 8м',
                'Бурение скважин': 'Бурение скважин', 'Погружение свай': 'Погружение свай',
                'Заполнение полости свай ЦПС': 'Заполнение трубопровода и подъем давления ',
                'Срезка свай, монтаж оголовков': 'Монтаж оголовков',
                'Засыпка пространства между сваями щебнем фракции 15-20 мм': 'Засыпка георешетки щебнем',
                'Монтаж деформационных марок': 'Монтаж деформационной марки (датчика деформации)',
                'Монтаж траверс': 'Монтаж траверс',
                'Монтаж м/к (связи, стойки, упоры, подкосы)': 'Монтаж м/к основных',
                'Устройство термометрических скважин': 'Устройство термометрических скважин',
                'Монтаж термометрических трубок': 'Монтаж термометрической трубки из трубы Ø 57х3,5мм',
                'Устройство песчанной подушки': 'Устройство песчаного основания с послойным уплотнением (вручную)',
                'Укладка габионов': 'Укладка георешетки',
                'Заполнение габионов щебнем фракции 70-120мм': 'Засыпка георешетки щебнем',
                'Монтаж термостабилизаторов': 'Монтаж термостабилизаторов', 'АКЗ свай и металлоконструкций': 'АКЗ МК',
                'Монтаж опор DN 500 (неподвижная, направляющая, скользящая)': 'Монтаж опор',
                'Сварка газопровода 530х24 мм': 'Сварка трубопровода',
                'Надземная прокладка газопровода 530х24 мм': 'Прокладка трубопроводов наружных инженерных систем из стальных труб',
                'Монтаж опор метанолопровода': 'Монтаж опор', 'Сварка метанолопровода 57х6мм': 'Сварка трубопровода',
                'Надземная прокладка метанолопровода 57х6мм': 'Прокладка трубопроводов наружных инженерных систем из стальных труб',
                'Монтаж заземления': 'Монтаж активного соляного заземления',
                'Очистка полости трубопроводов': 'Очистка полости трубопровода',
                'Гидроиспытания трубопроводов': 'Гидроиспытания трубопровода Ду 1400 мм',
                'Изоляция сварных соединений газопровода': 'Изоляция сварного стыка манжетами'}

new_dormitory_dict = {'АС/поз.1.1/Установка свай с заполнением': 'Погружение свай',
                      'АС/поз.1.1/Изготовление свай (наращивание)': 'Изготовление и монтаж оголовка металлической сваи',
                      'АС/поз.1.1/Монтаж оголовков': 'Монтаж оголовков',
                      'АС/поз.1.1/Монтаж ростверка': 'Монтаж ростверков и опорных конструкций под порталы, опоры ВЛ',
                      'АС/поз.1.1/АКЗ боковой поверхности металлических свай': 'АКЗ боковой поверхности металических свай',
                      'АС/поз.1.1/АКЗ м/к надземной части свай и оголовков, ростверка': 'АКЗ боковой поверхности металических свай',
                      'АС.ТТ/поз.1.1/Монтаж блоков общежития': 'Монтаж блоков ФБС',
                      'АС.ТТ/поз.1.1/Устройство внутренних лестниц': 'Монтаж лестниц',
                      'АС.ТТ/поз.1.1/Монтаж ферм стропильных': 'Монтаж м/к основных',
                      'АС.ТТ/поз.1.1/Монтаж нащельников на венткамеры': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж прогонов': 'Монтаж м/к основных',
                      'АС.ТТ/поз.1.1/Монтаж нащельников, элементов примыкания': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж крепления радиальных вентиляторов': 'Монтаж электроприборов',
                      'АС.ТТ/поз.1.1/Монтаж крепления наружных воздуховодов Д 250, 315, 800х400, 1000х600': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/Монтаж балок пола': 'Монтаж сборных конструкции полов',
                      'АС.ТТ/поз.1.1/Монтаж хомутов для крепления внутренних воздуховодов (27 шт.)': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/Монтаж крепежных элементов воздуховодов чердака (230 шт)': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/Монтаж крыльца №1': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж крыльца №3': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж крыльца №2': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж профилей обрамления проемов (3 шт)': 'Заполнение оконных проемов из ПВХ профилей',
                      'АС.ТТ/поз.1.1/Монтаж люка-лаза на кровлю': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж подставки под наружный блок кондиционера': 'Монтаж блоков ФБС',
                      'АС.ТТ/поз.1.1 Устройство оконных проемов': 'Заполнение оконных проемов из ПВХ профилей',
                      'АС.ТТ/поз.1.1 Устройство дверных проемов': 'Монтаж ворот, входных дверей',
                      'АС.ТТ/поз.1.1/Устройство подкровельной антиконденсатной пленки': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж профлиста (крыша, фронтон, козырек, цоколь)': 'Монтаж кровельного покрытия из профлиста',
                      'АС.ТТ/поз.1.1/Устройство утеплителя рулонного, жесткого': 'Монтаж кровли',
                      'АС.ТТ/поз.1.1/Монтаж панелей кровельных': 'Монтаж сэндвич-панелей',
                      'АС.ТТ/поз.1.1/Теплоизоляция техподполья': 'Устройство теплоизоляции',
                      'АС.ТТ/поз.1.1/Внутренняя отделка потолка "Армстронг"': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Внутренняя отделка ЦСП толщ.10 мм': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Внутренняя отделка стен "Бук"': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Устройство пола венткамер ЦСП толщ.10 мм и 16 мм': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Внутренняя отделка стен фибропанелью': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Внутренняя отделка потолка плитой перфорированной': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Устройство пола венткамер линолеум': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Внутренняя отделка плинтуса': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Внутренняя отделка пола линолеум': 'Внутренняя отделка',
                      'АС.ТТ/поз.1.1/Водоснабжение, монтаж трубопровода': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Водоснабжение, монтаж опор': 'Монтаж опор',
                      'АС.ТТ/поз.1.1/Водоснабжение, монтаж хомутов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Водоснабжение, монтаж муфт, угольников, тройников': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Водоснабжение, монтаж кранов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Теплоизоляция магистрального водопровода, монтаж теплоизоляции': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Канализация выше отм. +0.000, монтаж трубопровода': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Канализация ниже отм. -0.000, монтаж трубопровода': 'Прокладка импульсных трубопроводов ',
                      'Подземная прокладка трубопроводов К1, с учетом монтажа греющего кабеля и устройством теплоизоляции': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Монтаж стальных колодцев бытовой самотечной канализации К1, с учетом внутренней обвязки': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/Канализация выше отм. +0.000, монтаж муфт, отводов, тройников, ревизии, перехода': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Канализация выше отм. +0.000, монтаж клапанов воздушных': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Канализация ниже отм. -0.000, монтаж отводов, тройников, ревизии': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Теплоизоляция канализации, монтаж теплоизоляции': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/Прохождение труб через стык блок-модулей': 'Монтаж модулей / блок-боксов',
                      'АС.ТТ/поз.1.1/Крепление канализации под зданием': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/Противопожарный трубопровод, монтаж трубопровода': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Противопожарный трубопровод, монтаж тройников, отводов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Противопожарный трубопровод, монтаж шкафа пожарного в комплекте с огнетушителем, рукавом пожарным': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Противопожарный трубопровод, монтаж задвижек': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Обвязка бойлера, монтаж трубопровода': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Обвязка бойлера, отводов, тройников, ревизии, перехода, ниппель': 'Трубопроводы обвязки утилизационного теплообменника',
                      'АС.ТТ/поз.1.1/Обвязка бойлера, монтаж оборудования (водонагреватель, расширит. бак, регулятор давления, насос)': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/Обвязка бойлера, монтаж кранов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Обвязка бойлера, монтаж опор': 'Монтаж опор',
                      'АС.ТТ/поз.1.1/Общеобменная вентиляция, монтаж воздуховодов': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/В-8, монтаж воздуховода': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/Дымоудаление, прокладка воздуховодов': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/П-1,2,В12-14.1, монтаж воздуховодов': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/Дымоудаление, огнезащита воздуховодов': 'Огнезащита',
                      'АС.ТТ/поз.1.1/В-8, монтаж переходника, тройника, зонта': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/П-1, В1/1, В1/2, ВП1, ВД1, монтаж клапанов, ниппель': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/П-1,2,В12-14.1, монтаж шумоглушителя, зонта': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Дымоудаление, монтаж переходников, отводов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/П-1,2,В12-14.1, монтаж тройников, отводов, переходников, врезки': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Общеобменная вентиляция, монтаж вентилятора': 'Монтаж системы приточной вентиляции',
                      'АС.ТТ/поз.1.1/П-1,2,В12-14.1, монтаж вентрешетки': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/П-1,2,В12-14.1, монтаж крестовины, заглушек, хомутов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/П-1 монтаж приточной установки': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/Дымоудаление, монтаж клапанов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/П-1,В12-14.1, монтаж комплекта автоматики': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/П-1,2,В12-14.1, монтаж воздушно-тепловой завесы': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/Дымоудаление, монтаж зонтов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/П-1,2,В12-14.1, монтаж изоляции воздуховодов': 'Монтаж воздуховодов',
                      'АС.ТТ/поз.1.1/П-1.2.В12-14.1. монтаж сплит-системы': 'Монтаж оборудования системы кондиционирования (сплит-системы)',
                      'АС.ТТ/поз.1.1/Отопление (П1), монтаж трубопровода': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Отопление (ИТП), монтаж трубопроводов': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Отопление, монтаж трубопровода': 'Прокладка импульсных трубопроводов ',
                      'АС.ТТ/поз.1.1/Отопление, монтаж переходов, тройников, уголков, муфт': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Отопление, монтаж опор для труб': 'Монтаж опор',
                      'АС.ТТ/поз.1.1/Отопление (П1), монтаж кранов': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Отопление (ИТП), монтаж счетчиков, кранов, манометров, термометров и др.': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Отопление, монтаж радиаторов': 'Монтаж трудопроводов отопления',
                      'АС.ТТ/поз.1.1/Отопление (ИТП), монтаж опор для труб': 'Монтаж опор',
                      'АС.ТТ/поз.1.1/Отопление, монтаж насоса (ИТП), блока питания': 'Монтаж блоков ФБС',
                      'АС.ТТ/поз.1.1/Автоматика к вентиляторам, монтаж шкафов': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/Отопление (ИТП), монтаж переходов, тройников, уголков, муфт': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/Отопление, монтаж изоляции': 'Монтаж трудопроводов отопления',
                      'АС.ТТ/поз.1.1/Отопление (ИТП), монтаж изоляции': 'Монтаж трудопроводов отопления',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж кабеля': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж короба, кабель-канала, перегородки': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/Электрообогрев линии канализации до КНС': 'Водопровод противопожарный',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж светильников': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж ламп': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж ящика трансформаторного понижающего': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж выключателей, розеток': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж знаков безопасности': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж конвектора': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж обогревателя': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж футляра': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж трубы ПВХ с зондом': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/ЭОМ, монтаж металлорукава': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/АОВ, монтаж кабеля': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/АОВ, монтаж щита управления приточной установкой': 'Монтаж оборудования',
                      'АС.ТТ/поз.1.1/АОВ, монтаж щита управления вытяжкой': 'Монтаж шкафа управления электрообогревом',
                      'АС.ТТ/поз.1.1/АОВ, монтаж щита упр-я вентилятором дымоудаления, подпора': 'Монтаж стоек',
                      'АС.ТТ/поз.1.1/АОВ, монтаж блока упр-я клапанами': 'Монтаж щита пожарного с комплектующими',
                      'АС.ТТ/поз.1.1/АОВ, монтаж щита пожарной автоматики': 'Монтаж шкафа управления электрообогревом',
                      'АС.ТТ/поз.1.1/АОВ, монтаж пульта контроля и управления, блока контроля и индикации': 'Монтаж блоков ФБС',
                      'АС.ТТ/поз.1.1/АОВ, монтаж блока питания': 'Монтаж блоков ФБС',
                      'АС.ТТ/поз.1.1/АОВ, монтаж трубы ПВХ с зондом': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/АОВ, монтаж металлорукава': 'Монтаж ёмкости',
                      'АС.ТТ/поз.1.1/АОВ, монтаж лотка': 'Монтаж ёмкости',
                      'СС/поз.1.1/СБП. Монтаж кабеля': 'Монтаж силового кабеля',
                      'СС/поз.1.1/ПС. Монтаж кабеля': 'Монтаж силового кабеля',
                      'СС/поз.1.1/СКС. Установка коробов': 'Монтаж оборудования',
                      'СС/поз.1.1/СКС. Монтаж кабеля': 'Монтаж силового кабеля',
                      'СС/поз.1.1/ГГС. Монтаж кабеля': 'Монтаж силового кабеля',
                      'СС/поз.1.1/ГГС. Устройство трубы гофрированной с держателями': 'Монтаж ёмкости',
                      'СС/поз.1.1/ГГС. Установка распределительных коробов': 'Монтаж оборудования',
                      'СС/поз.1.1/СКС. Монтаж блока оптического': 'Монтаж блоков ФБС',
                      'СС/поз.1.1/СКС. Монтаж адаптеров, пигтейл, патч корд': 'Монтаж щита пожарного с комплектующими',
                      'СС/поз.1.1/ЛВС. Установка шкафа Rittal': 'Монтаж оборудования',
                      'СС/поз.1.1/ЛВС. Установка коммутаторов и модуля': 'Монтаж оборудования',
                      'СС/поз.1.1/СБП. Монтаж источника бесперебойного питания': 'Преобразователь, блоки питания, источники бесперебойного питания',
                      'СС/поз.1.1/СБП. Монтаж карты управления и мониторинга': 'Монтаж шкафа управления электрообогревом',
                      'СС/поз.1.1/СКС. Установка коробок с розеткой': 'Монтаж щита пожарного с комплектующими',
                      'СС/поз.1.1/ЛВС. Электрическая проверка и настройка': 'Проверка на герметичность ЗПТ',
                      'СС/поз.1.1/СТС. Установка аппаратов телефонных': 'Монтаж оборудования',
                      'СС/поз.1.1/СТС. Проверка и настройка АТС': 'Проверка на герметичность ЗПТ',
                      'СС/поз.1.1/ГГС. Установка громкоговорителей, IP- пульта': 'Монтаж оборудования',
                      'СС/поз.1.1/ГГС. Монтаж ящик распределительный Rittal': 'Монтаж ёмкости',
                      'Монтаж МК крепления Профлиста цоколь (Швеллер 100*50*4)': 'Монтаж кровельного покрытия из профлиста',
                      'Монтаж кронштейнов Кр1': 'Монтаж ёмкости',
                      'Зашивка цоколя профлистом С8-1150-0,7': 'Монтаж кровельного покрытия из профлиста',
                      'Монтаж цветных кассет на фасаде здания': 'Монтаж щита пожарного с комплектующими',
                      'Монтаж нащельников и герметизация': 'Монтаж кровли',
                      'Демонтаж нащельников, с учетом герметика': 'Монтаж кровли',
                      'Монтаж светильников на фасаде Глобус G300 - 8 шт., двухсторонний - 8 шт.': 'Монтаж щита пожарного с комплектующими',
                      'Монтаж светового короба СК-8 "Общежитие №3"': 'Монтаж лотков и коробов',
                      'Монтаж м/к (решетки, ограждения)': 'Монтаж щита пожарного с комплектующими',
                      'Демонтаж м/к (решетки, ограждения)': 'Монтаж щита пожарного с комплектующими',
                      'Монтаж МК крепления Плофлиста цоколя': 'Монтаж кровельного покрытия из профлиста',
                      'Зашивка цоколя профлистом': 'Монтаж кровельного покрытия из профлиста',
                      'Подшивка козырька': 'Электрические проводки в щитах и пультах шкафных и панельных',
                      'Монтаж решёток для витражей': 'Монтаж щита пожарного с комплектующими',
                      'АКЗ МК входной группы': 'АКЗ боковой поверхности металических свай',
                      'Монтаж Профлиста фронтона С8-1150-0.7': 'Монтаж кровельного покрытия из профлиста',
                      'Монтаж гофры': 'Монтаж ёмкости',
                      'Монтаж каб.канал': 'Монтаж ёмкости',
                      'Монтаж кабеля 5х6': 'Монтаж силового кабеля',
                      'Монтаж кабеля 3х2.5': 'Монтаж силового кабеля'}


@contextmanager
def elapsed_timer(lbl, is_print):
    start = default_timer()
    elapser = lambda: default_timer() - start
    yield lambda: elapser()
    end = default_timer()
    elapser = lambda: end - start
    if is_print:
        print(f'{lbl}: {round(end - start, 2)} sec')


def project_root() -> Path:
    """Returns project root folder."""
    return Path(__file__).parent.parent



import sys
import math
import scipy.stats as stats
from bamt.networks.continuous_bn import ContinuousBN


class ResTimeModel:
    def __init__(self, dbwrapper):
        self.wrapper = dbwrapper

    @staticmethod
    def nearest_value(precalculation: dict, volume_for_work: float, work_name: str) -> float:
        volumes = precalculation[work_name]
        result = min(volumes, key=lambda volume: abs(volume - volume_for_work))
        return result

    def get_resources_volumes(self, work_name, work_volume, resource_name=None) -> dict:
        precalc = self.wrapper.get_precalculation([work_name])
        if not precalc:
            return {}
        if work_volume == 0:
            volume_for_work = precalc[work_name].keys()[0]
            res = precalc[work_name][volume_for_work]['10%'].keys() if resource_name is None else [resource_name]
            worker_reqs = {}
            worker_reqs['worker_reqs'] = []
            for r in res:
                worker_reqs['worker_reqs'].append({'kind': r,
                                               'volume': 0,
                                               'min_count': 0,
                                               'max_count': 0})
            return worker_reqs
        worker_reqs = {}
        worker_reqs['worker_reqs'] = []
        volume_for_work = self.nearest_value(precalc, work_volume, work_name)
        res = precalc[work_name][volume_for_work]['10%'].keys() if resource_name is None else [resource_name]
        for r in res:
            resource_data = precalc[work_name][volume_for_work]
            min_res_value = math.ceil(resource_data['10%'][r])
            max_res_value = math.ceil(resource_data['90%'][r])
            volume = math.ceil(resource_data['50%'][r])
            min_res_value = max(min_res_value, 1)
            max_res_value = max(max_res_value, 1)
            volume = max(volume, 1)
            worker_reqs['worker_reqs'].append({'kind': r,
                                               'volume': volume,
                                               'min_count': min_res_value,
                                               'max_count': max_res_value})
        return worker_reqs

    def get_time(self, work_volume: dict, resources: dict, quantile: str) -> float:
        q = 0.5
        if quantile == '0.9':
            q = 0.1
        if quantile == '0.1':
            q = 0.9
        work_name = next(iter(work_volume))
        bn_params = self.wrapper.get_models(work_name)
        model_work_name, model_res_names = next((k[1], [r[0] for r in bn_params['edges']]) for k in bn_params['edges'])
        bn = ContinuousBN()
        bn.load(bn_params)
        test_data = {res_bn_name: resources[res_name] for res_bn_name in model_res_names for res_name in resources if res_name in res_bn_name}
        mu, var = bn.get_dist(model_work_name, test_data)
        mu = math.ceil(mu)
        prod = mu if var == 0 else work_volume[work_name] / 5 if math.isnan(mu) else stats.norm.ppf(q=q, loc=mu, scale=var)
        prod = max(prod, mu)
        prod = prod if prod != 0 else work_volume[work_name] / 5
        return max(math.ceil(work_volume[work_name] / prod), 1) + 1

    def estimate_time(self, work_unit, worker_list, mode='0.5'):
        if not worker_list:
            return 0
        if work_unit['volume'] == 0:
            return 0
        work_name = work_unit['name']
        work_volume = work_unit['volume']
        res_dict = {req['name']: req['_count'] for req in worker_list}
        return self.get_time(work_volume={work_name: work_volume}, resources=res_dict, quantile=mode) 



import os
from idbadapter.schedule_loader import Schedules, GRANULARY, TYPEDLVL2, PROCESSED
from idbadapter import MschmAdapter
import pandas as pd
from datetime import datetime, timedelta


class DBWrapper:
    def __init__(self, mschm_adapter, adapter, unit='', level=GRANULARY):
        self.mschm_adapter = mschm_adapter
        self.adapter = adapter
        self.level = level
        self.unit = unit

    def get_works(self):
        works = self.adapter.get_works_names(work_type=self.level)
        return works

    def get_resources(self):
        res = self.adapter.get_resources_names(res_type=GRANULARY)
        return res

    def get_act_names(self):
        df = self.adapter.get_all_works_name()
        return df

    def get_res_names(self):
        df = self.adapter.get_all_resources_name()
        return df

    def get_models(self, name_of_works: str):
        return self.mschm_adapter.get_model(name_of_works)

    def save_model(self, model: dict):
        self.mschm_adapter.save_model_to_db(model=model)

    def get_precalculation(self, name_of_works: list[str]):
        return self.mschm_adapter.get_precalculation(name_of_works=name_of_works)

    def save_precalculation(self, precalc: dict):
        self.mschm_adapter.save_precalculation_to_db(data=precalc)

    def delete_model(self, work_name):
        self.mschm_adapter.delete_model(work_name=work_name)

    def get_data(self, works_names, res_names):
        frames = []
        for p in self.adapter.from_names(
                works=works_names,    # список работ
                resources=res_names,  # список ресурсов
                ceil_limit=-1,        # ограничение по количеству строк на один запрос (-1 - выдать все)
                objects_limit=-1,     # ограничение на кол-во одновременно выдаваемых объектов (-1 - выдать все)
                crossing=False,       # переключение логики выбора объектов по работам и ресурсам (True - И, False - ИЛИ)
                key=self.level
        ):
            frames.append(p)

        data = pd.concat(frames)
        #data = data.loc[data['measurement_unit'].isin(self.unit+['-'])]
        

        # This part is tightly dependent on schedule_loader
        
        data.loc[data[self.level["column"]] == '-', self.level["column"]] = data.loc[
            data[self.level["column"]] == '-', 'name']
 
        return data



import json
import numpy as np
from sklearn.linear_model import LinearRegression



class JournalModel:
    def __init__(self, root, leaf, mapper):
        # if root in gas_ved_dict:
        #     root = gas_ved_dict[root]
        # if root in new_dormitory_dict:
        #     root = new_dormitory_dict[root]
        self.root = root
        self.leaf = leaf
        self.mapper = mapper

    @staticmethod
    def get_journal():
        with open('D:/GPN_KIP/auxiliary_data/journal_mapped.json') as f:
            journal = json.load(f)
        return journal

    def fit(self, hours=11.0):
        pass

    def predict(self, data):
        pass


class WorkJournal(JournalModel):
    def __init__(self, root, leaf, mapper):
        root = root.split("_")[0]
        leaf = leaf.split("_")[0]
        super().__init__(root=root, leaf=leaf, mapper=mapper)
        self.model = None

    def fit(self, hours=11.0):
        journal = self.get_journal()
        coef = journal[self.mapper[self.root][0]]["resources"][self.leaf]["number"] / \
               journal[self.mapper[self.root][0]]["resources"][self.leaf]["productivity"]
        #coef /= hours
        regressor = LinearRegression(fit_intercept=False)
        regressor.coef_ = np.array([coef])
        regressor.intercept_ = np.array([0.0])
        self.model = regressor

    def predict(self, data):
        return self.model.predict(data)


class PerformanceJournal(JournalModel):
    def __init__(self, root, leaf, mapper):
        super().__init__(root=root, leaf=leaf, mapper=mapper)
        self.hours = None
        self.productivity = None
        self.numbers = None

    def fit(self, hours=11.0):
        journal = self.get_journal()

        numbers = {}
        for root in self.root:
            numbers[root] = journal[self.mapper[self.leaf][0]]["resources"][root]["number"]
        self.numbers = numbers
        self.productivity = journal[self.mapper[self.leaf][0]]["resources"][self.root[0]]["productivity"]
        self.hours = hours

    def predict(self, data):
        min_numbers = []
        for i, node in enumerate(self.root):
            min_number = data[0, i] / self.numbers[node]
            min_numbers.append(min_number)
        coef = min(min_numbers)
        performance = [coef * self.productivity]

        return performance



import json

import pandas as pd
from sklearn.preprocessing import PolynomialFeatures
from sklearn.linear_model import LinearRegression
from sklearn.pipeline import Pipeline as Pipe

from idbadapter.schedule_loader import Schedules, GRANULARY, TYPEDLVL2, PROCESSED
from idbadapter import MschmAdapter
from stairsres.DBWrapper import DBWrapper
from dataset.MSGDataset import MSGDataset
from dataset.WindowDataset import WindowDataset
from dataset.ObjectNameProcessor import ObjectNameProcessor
from bayesian.networks import EgoResourceNet, EgoWorkNet, EgoPerformanceNet
from core.core_utils import gas_ved_dict, new_dormitory_dict
from dataset.PreprocessedMSG import PreprocessedMSG
URL = "postgresql+psycopg2://testuser:pwd@10.32.15.30:25432/test"


# TODO: finish pipeline class
class Pipeline:
    def __init__(self, work_name, work_name_processed, work_vol, project):
        self.work_name = work_name
        self.work_name_processed = work_name_processed
        self.work_vol = work_vol
        self.res_names = None
        # Загружаем ведомость в поле journal
        with open('D:/GPN_KIP/auxiliary_data/journal_mapped.json') as f:
            journal = json.load(f)
        self.journal = journal
        if work_name_processed:
            journal_map = journal[work_name_processed]
            self.res_names = [res_name for res_name in journal_map["resources"]]
            self.adapter = Schedules(URL)
            self.mschm_adapter = MschmAdapter(URL)
            wrapper = DBWrapper(adapter=self.adapter, mschm_adapter=self.mschm_adapter)
            res_mapper = ObjectNameProcessor(wrapper=wrapper)
            res_mapper_dict = res_mapper.create_granulary_dict(self.res_names, name_type='res')
            res_names_processed = []
            change_columns = {work_name_processed: work_name}
            for res_name in res_mapper_dict:
                res_names_processed.append(res_mapper_dict[res_name][0])
                change_columns[res_mapper_dict[res_name][0]] = res_name
            self.res_names_processed = res_names_processed
            self.change_columns = change_columns
            self.meases = [journal_map["measurements"]]
        else:
            print('NO Work in Ved '+work_name)

        # if project == 'electroline':
        #     journal_map = journal[work_name]
        # elif project == 'dormitory':
        #     journal_map = journal[new_dormitory_dict[work_name]]
        # else:
        #     journal_map = journal[gas_ved_dict[work_name]]
        

    def get_subworks(self, res_name):
        works = []
        meases = []
        for work in self.journal:
            if res_name in self.journal[work]["resources"]:
                works.append(work)
                meases.append(self.journal[work]["measurements"])
        return works, meases

    @staticmethod
    def __transform_dict(dict_from_pd: dict) -> dict:
        """
        Transforming res and work names in precalculations from Window Dataset format to usual one
        """
        dict_to_db = {}
        for res_name in dict_from_pd:
            new_res_name = res_name.replace('__res_fact', '')
            dict_to_db[new_res_name] = dict_from_pd[res_name][0]

        return dict_to_db

    def __cut_dataset(self, msg_data):
        """
        Trying to avoid outliers
        """
        q_01, q_09 = msg_data[self.work_name].quantile(q=0.1), msg_data[self.work_name].quantile(q=0.9)
        msg_data = msg_data[q_01 <= msg_data[self.work_name]]
        msg_data = msg_data[msg_data[self.work_name] <= q_09]

        return msg_data

    def __delete_zero_cols(self, msg_data):
        """
        Sometimes you might get the dataset which consists of only zeros
        """
        for col in msg_data.columns:
            if col == self.work_name:
                continue
            else:
                if (msg_data[col] == 0.0).all():
                    del msg_data[col]
        msg_data.loc[len(msg_data)] = 0.0

        return msg_data

    def __rename_cols(self, msg_data):
        """
        Rename columns according to WindowDataset format
        """
        new_cols = {}
        for col in msg_data.columns:
            if col == self.work_name:
                new_col = col + "__act_fact"
            elif col == "date":
                new_col = "Unnamed: 0"
            else:
                new_col = col + "__res_fact"
            new_cols[col] = new_col
        msg_data = msg_data.rename(columns=new_cols)

        return msg_data

    def check_work_name(self, msg_data):
        """
        Sometimes you might get the dataset without work from DataBase
        """
        if self.work_name not in list(msg_data.columns):
            raise Exception(f"NO WORK DATA FOR {self.work_name} IN FIT PERFORMANCE")

    def check_columns(self, msg_data):
        """
        Sometimes you might get the dataset without resources from DataBase
        """
        if len(msg_data.columns) < 2:
            raise Exception(f"NUM OF COLUMNS IS NOT ENOUGH FOR {self.work_name}")

    def __get_node_names_for_bn(self, window_data):
        """
        Defining nodes for EgoWorkNet and EgoPerformanceNet
        """
        new_res_names = []
        for col in window_data.columns:
            if "_prod" in col:
                perf_leaf = col
            elif "_act_fact" in col:
                work_root = col
            elif self.work_name not in col:
                new_res_names.append(col)
        return work_root, perf_leaf, new_res_names

    # def precalculate(self):
    #     work_name, work_name_processed = self.work_name, self.work_name_processed
    #     res_names, main_meases = self.res_names, self.meases
    #     res_names_processed = self.res_names_processed
        #   TODO: after demo uncomment this block
        #
        # subworks_dict = {}
        # for res_name in res_names:
        #     subworks, meases = self.get_subworks(res_name)
        #     subworks_dict[res_name] = {
        #         "measurements": meases,
        #         "subworks": subworks
        #     }
        #
        # first_flag = True
        # third_data = pd.DataFrame()
        # for res_name in res_names:
        #     print(f"GETTING DATA FOR {res_name}")
        #     wrapper = DBWrapper(URL=URL, level=TYPEDLVL2, unit=subworks_dict[res_name]["measurements"])
        #     model = MSGDataset(work_names=subworks_dict[res_name]["subworks"], res_names=[res_name], dbwrapper=wrapper)
        #     data = model.collect()
        #     print("GOT DATA")
        #
        #     bn = EgoResourceNet(root=[work for work in data.columns if work != res_name],
        #                         leaves=[res_name])
        #     nodes = data.columns.tolist()
        #     info = {"types": {
        #         node: "cont" for node in nodes
        #     }, "signs": {
        #         node: "pos" for node in nodes
        #     }}
        #     bn.add_nodes(info)
        #     print("FITTING BAYESIAN NETWORK")
        #     bn.fit(data, silent=True)
        #
        #     path_to_save_bn = '../models/' + bn.name
        #     if not os.path.exists(path_to_save_bn):
        #         os.mkdir(path_to_save_bn)
        #     bn.save(bn_name=path_to_save_bn + '/' + bn.name + '.json', models_dir='./')
        #
        #     data_loader = ResDataset(res_names=[res_name], window_data=data, root_dir="../",
        #                              save_data=False, save_path="./sec.csv")
        #     print("COLLECTING SECONDARY DATA")
        #     second_data = data_loader.collect()
        #     if first_flag:
        #         third_data = pd.concat([third_data, second_data[second_data[work_name] != 0]
        #                                                        [work_name].reset_index(drop=True)], axis=1)
        #         first_flag = False
        #     third_data = pd.concat([third_data, second_data[second_data[work_name] != 0]
        #                                                    [res_name].reset_index(drop=True)], axis=1)

        # wrapper = DBWrapper(adapter=self.adapter, mschm_adapter=self.mschm_adapter, level=GRANULARY, unit=main_meases)
        # model = MSGDataset(work_names=[work_name_processed], res_names=res_names_processed, dbwrapper=wrapper)
        # third_data = model.collect()
        # third_data = third_data.rename(columns=self.change_columns)
        # if work_name not in list(third_data.columns):
        #     print(f"NO WORK DATA FOR {work_name} IN PRECALCULATE")
        #     return None
        # third_data = third_data[third_data[work_name] != 0.0]
        # q_01, q_09 = third_data[work_name].quantile(q=0.1), third_data[work_name].quantile(q=0.9)
        # third_data = third_data[q_01 <= third_data[work_name]]
        # third_data = third_data[third_data[work_name] <= q_09]
        # if len(third_data.columns) < 2:
        #     print('\n\n\n')
        #     print("No resources")
        #     print(work_name, end='\n\n\n')
        #     return None
        # for col in third_data.columns:
        #     if col == work_name:
        #         continue
        #     else:
        #         if (third_data[col] == 0.0).all():
        #             del third_data[col]
        # res_names = [col for col in third_data.columns if col != work_name]
        # third_data.loc[len(third_data)] = 0.0
        #
        # bn_05 = EgoWorkNet(root=[work_name], leaves=res_names)
        # bn_01 = EgoWorkNet(root=[work_name], leaves=res_names, quantile='0.1')
        # bn_09 = EgoWorkNet(root=[work_name], leaves=res_names, quantile='0.9')
        #
        # nodes = third_data.columns.tolist()
        # info = {"types": {
        #     node: "cont" for node in nodes
        # }, "signs": {
        #     node: "pos" for node in nodes
        # }}
        #
        # bn_05.add_nodes(info)
        # bn_01.add_nodes(info)
        # bn_09.add_nodes(info)
        # bn_05.fit(third_data, silent=True)
        # bn_01.fit(third_data, silent=True)
        # bn_09.fit(third_data, silent=True)
        #
        # precalculation = {}
        # min_grid = third_data[work_name].min()
        # max_grid = third_data[work_name].max()
        # n = 30
        # h = (max_grid - min_grid) / n
        # test_value = min_grid
        #
        # for _ in range(n):
        #     test_data = pd.DataFrame({work_name: [test_value]})
        #     res_pred_05 = bn_05.predict(test_data)[res_names].to_dict('list')
        #     res_pred_01 = bn_01.predict(test_data)[res_names].to_dict('list')
        #     res_pred_09 = bn_09.predict(test_data)[res_names].to_dict('list')
        #
        #     res_pred_05 = self.__transform_dict(res_pred_05)
        #     res_pred_01 = self.__transform_dict(res_pred_01)
        #     res_pred_09 = self.__transform_dict(res_pred_09)
        #
        #     precalculation_for_value = {
        #         "10%": res_pred_01,
        #         "50%": res_pred_05,
        #         "90%": res_pred_09,
        #         "Prob": 0.0
        #     }
        #
        #     precalculation[test_value] = precalculation_for_value
        #     test_value += h
        #
        # work_name = work_name.replace('/', '#')
        # with open(f'./precalc_{work_name}.json', 'w') as fp:
        #     json.dump({work_name: precalculation}, fp)
        #
        # # wrapper = DBWrapper(URL=URL)
        # # wrapper.save_precalculation({work_name: precalculation})
        #
        # return precalculation

    def fit_performance(self, ved_mapper):
        if self.res_names:
            work_name, work_name_processed = self.work_name, self.work_name_processed
            res_names, main_meases = self.res_names, self.meases
            res_names_processed = self.res_names_processed
            file = open('D:/GPN_KIP/auxiliary_data/journal_mapped.json')
            journal = json.load(file)
            print([work_name_processed])
            print(res_names_processed)
            # getting MSG Dataset
            wrapper = DBWrapper(adapter=self.adapter, mschm_adapter=self.mschm_adapter, level=GRANULARY, unit=main_meases)
            #model = MSGDataset(work_names=[work_name_processed], res_names=res_names_processed, dbwrapper=wrapper)
            model = PreprocessedMSG(key_work_name=[work_name_processed], res_names=res_names_processed,dbwrapper=wrapper, journal=journal)
            msg_data = model.collect()
            # if len(msg_data) <= 50:
            #     model = MSGDataset(work_names=[work_name_processed], res_names=res_names_processed, dbwrapper=wrapper,
            #                        delete_zero_res=False)
            #     msg_data = model.collect()
            # preprocessing MSG Dataset
            try:
                if msg_data is not None:
                    msg_data = msg_data.rename(columns=self.change_columns)
                    self.check_work_name(msg_data=msg_data)
                    msg_data = msg_data[msg_data[work_name] != 0.0]
                    msg_data = self.__cut_dataset(msg_data=msg_data)
                    self.check_columns(msg_data=msg_data)
                    msg_data = self.__delete_zero_cols(msg_data=msg_data)
                    msg_data = self.__rename_cols(msg_data=msg_data)
                    self.check_columns(msg_data=msg_data)
            except Exception as error:
                print("ERROR: ", error)
                return None
            

            if msg_data is not None and msg_data.shape[0] > 30:
                # getting Window Dataset
                window_data_wrapper = WindowDataset(msg_data=msg_data, min_window=2, max_window=15)
                window_data = window_data_wrapper.collect()

                work_root, perf_leaf, new_res_names = self.__get_node_names_for_bn(window_data=window_data)

                # init regressor for EgoPerformanceNet
                stages = [('polynomial', PolynomialFeatures(degree=3)),
                            ('mode', LinearRegression())]
                pipeline = Pipe(stages)

                bn_05_perf = EgoPerformanceNet(root=new_res_names, leaves=[perf_leaf], regressor=pipeline)
                bn_01_perf = EgoPerformanceNet(root=new_res_names, leaves=[perf_leaf], quantile='0.1', regressor=pipeline)
                bn_09_perf = EgoPerformanceNet(root=new_res_names, leaves=[perf_leaf], quantile='0.9', regressor=pipeline)

                bn_05 = EgoWorkNet(root=[work_root], leaves=new_res_names)
                bn_01 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.1')
                bn_06 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.6')
                bn_07 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.7')
                bn_08 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.8')
                bn_09 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.9')

                nodes = new_res_names + [perf_leaf]
                info = {"types": {
                    node: "cont" for node in nodes
                }, "signs": {
                    node: "pos" for node in nodes
                }}

                bn_05_perf.add_nodes(info)
                bn_01_perf.add_nodes(info)
                bn_09_perf.add_nodes(info)
                bn_05_perf.fit(window_data[bn_05_perf.root + bn_05_perf.leaves])
                bn_01_perf.fit(window_data[bn_05_perf.root + bn_05_perf.leaves])
                bn_09_perf.fit(window_data[bn_05_perf.root + bn_05_perf.leaves])

                work_name = work_name.replace('/', '#')
                work_name = work_name.replace('*', '#')
                work_name = work_name.replace('+', '#')
                work_name = work_name.replace('-', '#')
                work_name = work_name.replace('"', '#')
                bn_05_perf.save(bn_name=f'./{work_name}_05')
                bn_09_perf.save(bn_name=f'./{work_name}_09')
                bn_01_perf.save(bn_name=f'./{work_name}_01')

                nodes = new_res_names + [work_root]
                info = {"types": {
                    node: "cont" for node in nodes
                }, "signs": {
                    node: "pos" for node in nodes
                }}

                bn_01.add_nodes(info)
                bn_05.add_nodes(info)
                bn_06.add_nodes(info)
                bn_07.add_nodes(info)
                bn_08.add_nodes(info)
                bn_09.add_nodes(info)

                bn_01.fit(window_data[bn_01.root + bn_01.leaves], silent=True)
                bn_05.fit(window_data[bn_01.root + bn_01.leaves], silent=True)
                bn_06.fit(window_data[bn_01.root + bn_01.leaves], silent=True)
                bn_07.fit(window_data[bn_01.root + bn_01.leaves], silent=True)
                bn_08.fit(window_data[bn_01.root + bn_01.leaves], silent=True)
                bn_09.fit(window_data[bn_01.root + bn_01.leaves], silent=True)

                precalculation = {}
                min_grid = min(window_data[work_root].min(), self.work_vol)
                max_grid = max(window_data[work_root].max(), self.work_vol)
                n = 30
                h = (max_grid - min_grid) / n
                test_value = min_grid

                for _ in range(n):
                    test_data = pd.DataFrame({work_root: [test_value]})
                    res_pred_05 = bn_05.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_01 = bn_01.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_06 = bn_06.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_07 = bn_07.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_08 = bn_08.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_09 = bn_09.predict(test_data)[new_res_names].to_dict('list')

                    res_pred_05 = self.__transform_dict(res_pred_05)
                    res_pred_01 = self.__transform_dict(res_pred_01)
                    res_pred_06 = self.__transform_dict(res_pred_06)
                    res_pred_07 = self.__transform_dict(res_pred_07)
                    res_pred_08 = self.__transform_dict(res_pred_08)
                    res_pred_09 = self.__transform_dict(res_pred_09)

                    precalculation_for_value = {
                        "10%": res_pred_01,
                        "50%": res_pred_05,
                        "60%": res_pred_06,
                        "70%": res_pred_07,
                        "80%": res_pred_08,
                        "90%": res_pred_09,
                        "Prob": 0.0
                    }

                    precalculation[test_value] = precalculation_for_value
                    test_value += h

                work_name = work_name.replace('/', '#')
                with open(f'./precalc_{work_name}.json', 'w') as fp:
                    json.dump({work_name: precalculation}, fp)
            else:
                print('JOURNAL learning')
                work_root = work_name
                perf_leaf = work_name
                new_res_names = res_names

                
                

                # init regressor for EgoPerformanceNet
                stages = [('polynomial', PolynomialFeatures(degree=3)),
                            ('mode', LinearRegression())]
                pipeline = Pipe(stages)

                bn_05_perf = EgoPerformanceNet(root=new_res_names, leaves=[perf_leaf], regressor=pipeline, mapper=ved_mapper)
                bn_01_perf = EgoPerformanceNet(root=new_res_names, leaves=[perf_leaf], quantile='0.1', regressor=pipeline, mapper=ved_mapper)
                bn_09_perf = EgoPerformanceNet(root=new_res_names, leaves=[perf_leaf], quantile='0.9', regressor=pipeline, mapper=ved_mapper)

                bn_05 = EgoWorkNet(root=[work_root], leaves=new_res_names, mapper=ved_mapper)
                bn_01 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.1', mapper=ved_mapper)
                bn_06 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.6', mapper=ved_mapper)
                bn_07 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.7', mapper=ved_mapper)
                bn_08 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.8',mapper=ved_mapper)
                bn_09 = EgoWorkNet(root=[work_root], leaves=new_res_names, quantile='0.9',mapper=ved_mapper)

                nodes = new_res_names + [perf_leaf]
                info = {"types": {
                    node: "cont" for node in nodes
                }, "signs": {
                    node: "pos" for node in nodes
                }}

                bn_05_perf.add_nodes(info)
                bn_01_perf.add_nodes(info)
                bn_09_perf.add_nodes(info)
                bn_05_perf.fit(data=None)
                bn_01_perf.fit(data=None)
                bn_09_perf.fit(data=None)

                work_name = work_name.replace('/', '#')
                work_name = work_name.replace('*', '#')
                work_name = work_name.replace('+', '#')
                work_name = work_name.replace('-', '#')
                work_name = work_name.replace('"', '#')
                bn_05_perf.save(bn_name=f'./{work_name}_05')
                bn_09_perf.save(bn_name=f'./{work_name}_09')
                bn_01_perf.save(bn_name=f'./{work_name}_01')

                nodes = new_res_names + [work_root]
                info = {"types": {
                    node: "cont" for node in nodes
                }, "signs": {
                    node: "pos" for node in nodes
                }}

                bn_01.add_nodes(info)
                bn_05.add_nodes(info)
                bn_06.add_nodes(info)
                bn_07.add_nodes(info)
                bn_08.add_nodes(info)
                bn_09.add_nodes(info)

                bn_01.fit(data=None)
                bn_05.fit(data=None)
                bn_06.fit(data=None)
                bn_07.fit(data=None)
                bn_08.fit(data=None)
                bn_09.fit(data=None)

                precalculation = {}
                min_grid = self.work_vol
                max_grid = 30*self.work_vol
                n = 30
                h = (max_grid - min_grid) / n
                test_value = min_grid

                for _ in range(n):
                    test_data = pd.DataFrame({work_root: [test_value]})
                    res_pred_05 = bn_05.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_01 = bn_01.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_06 = bn_06.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_07 = bn_07.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_08 = bn_08.predict(test_data)[new_res_names].to_dict('list')
                    res_pred_09 = bn_09.predict(test_data)[new_res_names].to_dict('list')

                    res_pred_05 = self.__transform_dict(res_pred_05)
                    res_pred_01 = self.__transform_dict(res_pred_01)
                    res_pred_06 = self.__transform_dict(res_pred_06)
                    res_pred_07 = self.__transform_dict(res_pred_07)
                    res_pred_08 = self.__transform_dict(res_pred_08)
                    res_pred_09 = self.__transform_dict(res_pred_09)

                    precalculation_for_value = {
                        "10%": res_pred_01,
                        "50%": res_pred_05,
                        "60%": res_pred_06,
                        "70%": res_pred_07,
                        "80%": res_pred_08,
                        "90%": res_pred_09,
                        "Prob": 0.0
                    }

                    precalculation[test_value] = precalculation_for_value
                    test_value += h

                work_name = work_name.replace('/', '#')
                with open(f'./precalc_{work_name}.json', 'w') as fp:
                    json.dump({work_name: precalculation}, fp)

        else:
            print('Problems with ' + self.work_name)
            return None

        # wrapper = DBWrapper(URL=URL)
        # wrapper.save_model({work_name + '_0.5': bn_05.get_params_for_db()})
        # wrapper.save_model({work_name + '_0.1': bn_01.get_params_for_db()})
        # wrapper.save_model({work_name + '_0.9': bn_09.get_params_for_db()})
